import gradio as gr
import groovy as gv
print(gv.__version__)
from flow import flow

with gr.Blocks() as app:
    task_box = gr.Textbox(label="🕺 Groovy Flow", value="flow.task")
    with gr.Row():
        for component in flow.inputs:
            component.render()

    @gr.on(
        triggers=[app.load] + [input.change for input in flow.inputs],
        inputs=flow.inputs,
        outputs=[prompt_box],
        trigger_mode="always_last",
    )
    def construct_prompt(*input_values):
        return prompt.format(*input_values)
                
    with gr.Row(scale=5):
        gr.Image()

if __name__ == "__main__":
    app.launch()



import pytest

import groovy

FAILED_VALIDATION_ERR_MESSAGE = "failed validation"


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "url",
    [
        "https://localhost",
        "http://127.0.0.1/file/a/b/c",
        "http://[::1]",
        "https://192.168.0.1",
        "http://10.0.0.1?q=a",
        "http://192.168.1.250.nip.io",
    ],
)
async def test_local_urls_fail(url):
    with pytest.raises(ValueError, match=FAILED_VALIDATION_ERR_MESSAGE):
        await groovy.get(url)


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "url",
    [
        "https://google.com",
        "https://8.8.8.8/",
        "http://93.184.215.14.nip.io/",
        "https://huggingface.co/datasets/dylanebert/3dgs/resolve/main/luigi/luigi.ply",
    ],
)
async def test_public_urls_pass(url):
    await groovy.get(url)


@pytest.mark.asyncio
async def test_domain_whitelist():
    try:
        await groovy.get(
            "http://192.168.1.250.nip.io", domain_whitelist=["192.168.1.250.nip.io"]
        )
    except ValueError as e:
        assert FAILED_VALIDATION_ERR_MESSAGE not in str(e)
    except Exception:
        pass  # Other exeptions (e.g. connection timeouts) are fine

    with pytest.raises(ValueError, match=FAILED_VALIDATION_ERR_MESSAGE):
        await groovy.get(
            "http://192.168.1.250.nip.io", domain_whitelist=["huggingface.co"]
        )


@pytest.mark.asyncio
async def test_transport_false():
    try:
        await groovy.get("http://192.168.1.250.nip.io", _transport=False)
    except ValueError as e:
        assert FAILED_VALIDATION_ERR_MESSAGE not in str(e)
    except Exception:
        pass  # Other exeptions (e.g. connection timeouts) are fine



import os
from collections.abc import Generator
from typing import TYPE_CHECKING, Callable, Sequence, Union

if TYPE_CHECKING:
    import gradio as gr


class Flow:
    """
    The core class in Groovy. A Flow consists of:
    * A task to run (string or format string)
    * A set of input components (optional)
    * A function that accepts a task string and yields an arbitrary number of `str`, `PIL.Image`, or `gr.ChatMessage` responses.

    After instantiating a Flow, you can call .launch() to launch the flow in a Gradio app.
    """

    def __init__(
        self,
        task: str,
        inputs: Sequence[gr.components.Component] | None = None,
        stream_fn: Callable[[str], Generator[Union[str, gr.ChatMessage], None, None]]
        | None = None,
    ):
        """
        Parameters:
            task: The task to run. Can be a regular string or a format string, in which case the input components' values will be passed to it.
            inputs: The input components whose values will be passed to the task, if it's a format string.
            stream_fn: The generator function that accepts a task string and yields an arbitrary number of `str`, `PIL.Image`, or `gr.ChatMessage` responses. If not provided, the default streamer (which browses the web to complete a task) will be used.
        """
        # Import here to speed up the import time of the groovy module
        from groovy.agent import browser_agent_streamer

        self.task = task
        self.inputs = inputs or []
        self.stream_fn = stream_fn or browser_agent_streamer

    def launch(self, artifacts_dir: str | None = None, run_immediately: bool = False):
        """
        Launches the flow in a Gradio app.

        Parameters:
            artifacts_dir: The directory to save artifacts (screenshots, etc.) to. If None, uses the working directory.
            run_immediately: Whether to run the task immediately at .launch() or whether to wait for the user to click the "Run" button in the Gradio app.
        """
        # Import here to speed up the import time of the groovy module
        from groovy.app import create_app

        self.artifacts_dir = artifacts_dir or os.getcwd()
        self.app = create_app(
            self,
            self.inputs,
            self.task,
            self.stream_fn,
            run_immediately,
            self.artifacts_dir,
        )
        _, self.url, _ = self.app.launch(inline=False, inbrowser=True)
        return self.url



import click
import os
import re
import huggingface_hub
import gradio as gr

from groovy.flow import Flow


@click.group()
def cli():
    pass


@cli.command()
def publish():
    """Publish a Groovy flow as a Gradio app"""
    click.echo(
        "This will create a new app.py file in the current directory and publish to Hugging Face Spaces\n"
    )
    flow_path = click.prompt("Path to flow file in current directory", default="flow.py")
    flow_name = click.prompt(
        "Name of the variable containing your Flow instance (e.g., 'my_flow' if you have 'my_flow = Flow()')",
        default="flow"
    )
    publish_all = click.confirm(f"Publish entire directory? (If no, only app.py, README.md, requirements.txt, and {flow_path} will be published)", default=False)

    # Convert relative path to module path
    module_path = flow_path.replace("/", ".").replace("\\", ".").rstrip(".py")

    # Create the app.py file
    app_content = f"""import gradio as gr
import groovy as gv
print(gv.__version__)
from {module_path} import {flow_name}

with gr.Blocks() as app:
    task_box = gr.Textbox(label="🕺 Groovy Flow", value="{flow_name}.task")
    with gr.Row():
        for component in {flow_name}.inputs:
            component.render()

    @gr.on(
        triggers=[app.load] + [input.change for input in {flow_name}.inputs],
        inputs={flow_name}.inputs,
        outputs=[prompt_box],
        trigger_mode="always_last",
    )
    def construct_prompt(*input_values):
        return prompt.format(*input_values)
                
    with gr.Row(scale=5):
        gr.Image()

if __name__ == "__main__":
    app.launch()
"""

    with open("app.py", "w") as f:
        f.write(app_content)

    
    repo_directory = os.getcwd()
    dir_name = os.path.basename(repo_directory)
    
    hf_api = huggingface_hub.HfApi()
    try:
        whoami = hf_api.whoami()
        if whoami["auth"]["accessToken"]["role"] != "write":
            click.echo("Need 'write' access token to create a Spaces repo.")
            huggingface_hub.login(add_to_git_credential=False)
    except OSError:
        click.echo("Need 'write' access token to create a Spaces repo.")
        huggingface_hub.login(add_to_git_credential=False)
    
    title = click.prompt("Enter Spaces app title", default=dir_name)
    title = format_title(title)
    
    click.echo(f"\n✨ Created app.py with `{flow_name}` from `{flow_path}`. Publishing...")
    readme_file = os.path.join(repo_directory, "README.md")
    configuration = {
        "title": title,
        "app_file": "app.py",
        "sdk": "gradio",
        "sdk_version": gr.__version__,
        "hardware": "cpu-basic"
    }
    huggingface_hub.metadata_save(readme_file, configuration)
    
    # Create space
    space_id = huggingface_hub.create_repo(
        configuration["title"],
        space_sdk="gradio",
        repo_type="space",
        exist_ok=True,
        space_hardware=configuration["hardware"],
    ).repo_id
    
    requirements_path = "requirements.txt"
    if not os.path.exists(requirements_path):
        with open(requirements_path, "w") as f:
            f.write("groovy\nhelium\n")
    
    if publish_all:
        hf_api.upload_folder(
            repo_id=space_id,
            repo_type="space",
            folder_path=repo_directory,
        )
    else:
        files_to_upload = ["app.py", flow_path, "README.md", "requirements.txt"]
        for file in files_to_upload:
            hf_api.upload_file(
                repo_id=space_id,
                repo_type="space",
                path_in_repo=file,
                path_or_fileobj=os.path.join(repo_directory, file),
            )
    
    click.echo(f"\n🚀 Space published at https://huggingface.co/spaces/{space_id}")

def format_title(title: str):
    """Format title to be compatible with Hugging Face Spaces naming requirements"""
    title = title.replace(" ", "_")
    title = re.sub(r"[^a-zA-Z0-9\-._]", "", title)
    title = re.sub("-+", "-", title)
    while title.startswith("."):
        title = title[1:]
    return title


@cli.command()
@click.argument("task")
def run(task: str):
    """Launch a Groovy flow with the specified task"""
    flow = Flow(task)
    flow.launch(run_immediately=True)


def main():
    cli()
